import type { ThemedToken, ThemeRegistration } from '@shikijs/types'
import type { ScrollViewProps, StyleProp, TextStyle, ViewStyle } from 'react-native'
import type { SyntaxHighlighterProps } from 'src/types/shiki'
import React, { useMemo } from 'react'
import { Platform, ScrollView, StyleSheet, Text, View } from 'react-native'
import { useSyntaxHighlighter } from '../../hooks/useSyntaxHighlighter'

const monospaceFont = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

function transformThemeToStyles(theme: ThemeRegistration) {
  const defaultBackground = theme.colors?.['editor.background']
  const foreground = theme.colors?.['editor.foreground']

  const styles = StyleSheet.create({
    scrollView: {
      flex: 1,
      minHeight: 20,
    },
    codeContainer: {
      padding: 16,
      minWidth: '100%',
    },
    lineContainer: {
      flexDirection: 'row',
      flexWrap: 'wrap',
    },
    baseToken: {
      fontFamily: monospaceFont,
    },
  })

  const colorStyles = new Map<string, TextStyle>()

  return {
    base: styles,
    background: defaultBackground,
    getTokenStyle: (token: ThemedToken): TextStyle => {
      const key = `${token.color || ''}-${token.fontStyle || ''}`

      if (!colorStyles.has(key)) {
        colorStyles.set(key, {
          color: token.color || foreground,
          fontStyle: token.fontStyle ? 'italic' : 'normal',
          fontWeight: token.fontStyle ? 'bold' : 'normal',
        })
      }

      return colorStyles.get(key)!
    },
  }
}

interface Props extends SyntaxHighlighterProps {
  scrollViewProps?: ScrollViewProps
  textStyle?: StyleProp<TextStyle>
  containerStyle?: StyleProp<ViewStyle>
}

const SyntaxHighlighter: React.FC<Props> = ({ text, language, languageId, theme, fontSize = 14, scrollViewProps, textStyle, containerStyle }) => {
  const { tokens } = useSyntaxHighlighter({
    text,
    language,
    languageId: languageId || String(language),
    theme,
  })

  const styles = useMemo(() => transformThemeToStyles(theme as ThemeRegistration), [theme])

  return (
    <ScrollView
      horizontal
      {...scrollViewProps}
      style={[styles.base.scrollView, scrollViewProps?.style]}
      contentContainerStyle={[styles.base.codeContainer, { backgroundColor: theme.colors?.['editor.background'] || '#1e1e1e' }, scrollViewProps?.contentContainerStyle, containerStyle]}
    >
      <View onStartShouldSetResponder={() => true}>
        {tokens.map((line, rowIndex) => (
          <View key={rowIndex} style={[styles.base.lineContainer, { backgroundColor: theme.colors?.['editor.background'] || '#1e1e1e' }]}>
            {line.map((token, tokenIndex) => (
              <Text key={`${rowIndex}-${tokenIndex}`} style={[styles.base.baseToken, styles.getTokenStyle(token), { fontSize }, textStyle]}>
                {token.content}
              </Text>
            ))}
          </View>
        ))}
      </View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
